==> reverie_dreams-0.4.6%2B1.21.7%2Bbeta-sources.jar: cc/thonly/polymer/mixin/ElementUpdateListenerMixin.java <==
package cc.thonly.polymer.mixin;

import cc.thonly.polymer.entity.bil.BlockbenchEntityHolder;
import de.tomalbrc.bil.core.extra.ElementUpdateListener;
import de.tomalbrc.bil.core.holder.base.AbstractAnimationHolder;
import de.tomalbrc.bil.core.holder.entity.EntityHolder;
import de.tomalbrc.bil.core.model.Pose;
import eu.pb4.polymer.virtualentity.api.VirtualEntityUtils;
import eu.pb4.polymer.virtualentity.api.elements.GenericEntityElement;
import net.minecraft.class_1297;
import net.minecraft.class_243;
import net.minecraft.class_3222;
import net.minecraft.class_3532;
import org.joml.Vector3f;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(value = ElementUpdateListener.class, remap = false)
public abstract class ElementUpdateListenerMixin {
    @Shadow protected abstract void updateEntityBasedHolder(EntityHolder<?> holder, Pose pose);

    @Shadow @Final protected GenericEntityElement element;

    @Inject(method = "update", at = @At(value = "INVOKE", target = "Lde/tomalbrc/bil/core/extra/ElementUpdateListener;updateNonEntityBasedHolder(Lde/tomalbrc/bil/core/holder/base/AbstractAnimationHolder;Lde/tomalbrc/bil/core/model/Pose;)V"), cancellable = true)
    public void updateInvoke(class_3222 serverPlayer, AbstractAnimationHolder holder, Pose pose, CallbackInfo ci) {
        if (holder instanceof BlockbenchEntityHolder<?,?> entityHolder) {
            this.updateBBEntityBasedHolder(entityHolder, pose);
            ci.cancel();
        }
    }

    @Unique
    private void updateBBEntityBasedHolder(BlockbenchEntityHolder<?,?> holder, Pose pose) {
        class_1297 entity = holder.getEntity();
        float scale = holder.getScale();
        float yRot = entity.method_36454();
        float angle = yRot * class_3532.field_29847;

        Vector3f offset = pose.translation();
        if (scale != 1F) {
            offset.mul(scale);
        }
        offset.rotateY(-angle);

        class_243 pos = holder.getPos().method_1031(offset.x, offset.y, offset.z);
        holder.sendPacket(VirtualEntityUtils.createMovePacket(
                this.element.getEntityId(),
                pos,
                pos,
                true,
                yRot,
                0F
        ));
    }
}

==> reverie_dreams-0.4.8%2B1.21.7%2Bbeta-sources.jar: cc/thonly/polymer/mixin/ElementUpdateListenerMixin.java <==
package cc.thonly.polymer.mixin;

import cc.thonly.polymer.entity.bil.OverlayEntityHolder;
import de.tomalbrc.bil.core.extra.ElementUpdateListener;
import de.tomalbrc.bil.core.holder.base.AbstractAnimationHolder;
import de.tomalbrc.bil.core.holder.entity.EntityHolder;
import de.tomalbrc.bil.core.model.Pose;
import eu.pb4.polymer.virtualentity.api.VirtualEntityUtils;
import eu.pb4.polymer.virtualentity.api.elements.GenericEntityElement;
import net.minecraft.class_1297;
import net.minecraft.class_243;
import net.minecraft.class_3222;
import net.minecraft.class_3532;
import org.joml.Vector3f;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(value = ElementUpdateListener.class, remap = false)
public abstract class ElementUpdateListenerMixin {
    @Shadow protected abstract void updateEntityBasedHolder(EntityHolder<?> holder, Pose pose);

    @Shadow @Final protected GenericEntityElement element;

    @Inject(method = "update", at = @At(value = "INVOKE", target = "Lde/tomalbrc/bil/core/extra/ElementUpdateListener;updateNonEntityBasedHolder(Lde/tomalbrc/bil/core/holder/base/AbstractAnimationHolder;Lde/tomalbrc/bil/core/model/Pose;)V"), cancellable = true)
    public void updateInvoke(class_3222 serverPlayer, AbstractAnimationHolder holder, Pose pose, CallbackInfo ci) {
        if (holder instanceof OverlayEntityHolder<?,?> entityHolder) {
            this.updateBBEntityBasedHolder(entityHolder, pose);
            ci.cancel();
        }
    }

    @Unique
    private void updateBBEntityBasedHolder(OverlayEntityHolder<?,?> holder, Pose pose) {
        class_1297 entity = holder.getEntity();
        float scale = holder.getScale();
        float yRot = entity.method_36454();
        float angle = yRot * class_3532.field_29847;

        Vector3f offset = pose.translation();
        if (scale != 1F) {
            offset.mul(scale);
        }
        offset.rotateY(-angle);

        class_243 pos = holder.getPos().method_1031(offset.x, offset.y, offset.z);
        holder.sendPacket(VirtualEntityUtils.createMovePacket(
                this.element.getEntityId(),
                pos,
                pos,
                true,
                yRot,
                0F
        ));
    }
}
